﻿using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Timers;
using System.Linq;
using System.Threading.Tasks;
using System.ComponentModel.DataAnnotations.Schema;

namespace DiscoBot.gsmeet
{
    public class GSMeetContext : DbContext
    {
        private ulong guildId;
        public DbSet<DBSheet> GSheets { get; set; }

        public GSMeetContext(ulong guildId)
        {
            this.guildId = guildId;
        }

        protected override void OnConfiguring(DbContextOptionsBuilder optionsBuilder)
        {
            optionsBuilder.UseSqlite("Data Source=" + guildId + "-gsmeet.db");
        }
    }

    public class DBSheet
    {
        [Key]
        public string Name { get; set; }
        public string Id { get; set; }
        public string SheetName { get; set; }
        public ulong Channel { get; set; }
        public TimeSpan CheckInterval { get; set; }
        public DateTimeOffset LastChecked { get; set; }
        public List<GSMeetEventNotification> NotifiedEvents { get; set; }
        public List<GSMeetUserNotification> NotifiedDates { get; set; }
    }

    public class GSMeetEventNotification
    {
        [ScaffoldColumn(false)]
        [DatabaseGenerated(DatabaseGeneratedOption.Identity)]
        [Key]
        public long Id { get; set; }
        public DateTimeOffset Date { get; set; }
    }

    public class GSMeetUserNotification
    {
        [ScaffoldColumn(false)]
        [DatabaseGenerated(DatabaseGeneratedOption.Identity)]
        [Key]
        public long Id { get; set; }
        public string DiscordTag { get; set; }
        public DateTimeOffset Date { get; set; }
    }

    public class GSMeetSheetData
    {
        public GSMeetSheetData(DBSheet sheet)
        {
            this.Sheet = sheet;
        }
        public DBSheet Sheet { get; set; }
        public List<GSMeetUser> Users { get; set; }
        public List<DateTimeOffset> Dates { get; set; }
    }

    public class GSMeetUser
    {
        public GSMeetUser()
        {
            this.Signups = new Dictionary<DateTimeOffset, string>();
        }
        public string DiscordTag { get; set; }
        public string Name { get; set; }
        public Dictionary<DateTimeOffset, string> Signups { get; set; }
    }
}
